#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "connectionwrapper4xi.py">
#<abstract4xi abstract = "provides the class ConnectionWrapper4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
#<note4xi note = "ConnectionWrapper4Xi compared to ConnectionWrapper of connection.py and found no changes due to Rel 7.21"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ConnectionWrapper4Xi']
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import errno
    import time
    import tempfile
    import itertools         
except:
    pass
try:                 # rare ones
    import socket          
except:
    pass 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
#"
#/>
      
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>


# =================================================================================================
#<class4xi class = "ConnectionWrapper4Xi" base = "object" child = "None" synopsis = "support for using xmlrpclib for serialization" usage = "
#    connection.py     
#        XmlListener(Listener): 
#            return ConnectionWrapper(obj, _xml_dumps, _xml_loads)
#    xmllistener4xi.py:
#        return ConnectionWrapper4Xi(obj2xi, _xml_dumps, _xml_loads)
#    xml_dumps4xi.py:
#        return ConnectionWrapper(obj, _xml_dumps, _xml_loads)
#    _xml_loads4xi.py:
#        return ConnectionWrapper(obj, _xml_dumps, _xml_loads)"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class ConnectionWrapper4Xi(object):
    '''
    ConnectionWrapper4Xi class offers services to wrap connections
    '''
   
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('send', 'recv', 'sense', 'reason', 'ace')      
    #"
    #/>
                                 
                                     

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ConnectionWrapper4Xi" usage = "initializes with (self, conn, dumps, loads)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, conn, dumps, loads): 
        '''
        ConnectionWrapper4Xi __init__method initializes with (self, conn, dumps, loads)
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)           
        # -----------------------------------------------------------------------------------------
        class2print = 'ConnectionWrapper4Xi'
        method2print = "__init__"
        
        message2print = "{started" + "; init-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

    
        message2print = "{(Step 1/4a) borning" + "; connectionwrapper2xi-of-ConnectionWrapper4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       

        message2print = "{(Step 1/4b) inheriting" + "; object-inheriting-by-connectionwrapper2xi-of-ConnectionWrapper4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
                                               
        message2print = "{(Step 1/4b) passed" + "; conn-passed-to-init-of-ConnectionWrapper4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
                                          
        message2print = "{(Step 1/4c) passed" + "; dumps-passed-to-init-of-ConnectionWrapper4Xi: " + str(dumps) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
           
                                          
        message2print = "{(Step 1/4d) passed" + "; loads-passed-to-init-of-ConnectionWrapper4Xi: " + str(loads) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)           
        # -----------------------------------------------------------------------------------------
        self._conn = conn
        
        message2print = "{(Step 2/4a) set" + "; conn-of-connectionwrapper2xi-set-by-init-of-ConnectionWrapper4Xi to: " + str(self._conn)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                
        
        self._dumps = dumps
        
        message2print = "{(Step 2/4b) set" + "; dumps-of-connectionwrapper2xi-set-by-init-of-ConnectionWrapper4Xi to: " + str(self._dumps)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
           
        
        self._loads = loads
        
        message2print = "{(Step 2/4c) set" + "; loads-of-connectionwrapper2xi-set-by-init-of-ConnectionWrapper4Xi to: " + str(self._loads)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)           
        # -----------------------------------------------------------------------------------------                        
        for attr2xi in ('fileno', 'close', 'poll', 'recv_bytes', 'send_bytes'):
            obj2xi = getattr(conn, attr2xi)
            
            message2print = "{(Step 3/4a) set" + "; obj2xi-set-by-init-of-ConnectionWrapper4Xi to: " + str(obj2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                    
            
            message2print = "{(Step 3/4b) passing" + "; attr2xi-passing-to-setattr-by-init-of-ConnectionWrapper4Xi: " + str(attr2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            
            message2print = "{(Step 3/4c) passing" + "; obj2xi-passing-to-setattr-by-init-of-ConnectionWrapper4Xi: " + str(obj2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
                       
                    
            
            setattr(self, attr2xi, obj2xi)
            
            
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)           
        # -----------------------------------------------------------------------------------------        

        message2print = "{(Step 4/4a) activated" + "; connectionwrapper2xi-of-ConnectionWrapper4Xi-activated: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 4/4b) inheriting" + "; object-inheriting-by-connectionwrapper2xi-of-ConnectionWrapper4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
              
        message2print = "{(Step 4/4c) returning" + "; init-of-ConnectionWrapper4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
                                 
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "send-of-ConnectionWrapper4Xi" usage = "sends with (self, obj)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def send(self, obj):
        '''
        ConnectionWrapper4Xi send method: sends with (self, obj)
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)          
        # -----------------------------------------------------------------------------------------        
           
               
        class2print = 'ConnectionWrapper4Xi'        
        method2print = "send"
        
        
        message2print = "{(Step 1/3a) started" + "; send-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
        message2print = "{(Step 1/3b) passed" + "; object-passed-to-send-of-ConnectionWrapper4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)           
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 2/3a) passing" + "; object-passing-to-dumps-of-connectionwrapper2xi-by-send-of-ConnectionWrapper4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        

        s2xi = self._dumps(obj)
        
        message2print = "{(Step 2/3b) set" + "; s2xi-set-by-send-of-ConnectionWrapper4Xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
        message2print = "{(Step 2/3c) passing" + "; s2xi-passing-to-conn-send-bytes-of-connectionwrapper2xi-by-send-of-ConnectionWrapper4Xi: " + str(s2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        message2print = "{(Step 2/3d) calling" + "; conn-send-bytes-of-connectionwrapper2xi-calling-by-send-of-ConnectionWrapper4Xi: " + str(self._conn.send_bytes) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
        self._conn.send_bytes(s2xi)
        
        message2print = "{(Step 2/3e) performed" + "; conn-send-bytes-of-connectionwrapper2xi-performed-by-send-of-ConnectionWrapper4Xi: " + str(self._conn.send_bytes) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                          
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3/3) returning" + "; send-of-ConnectionWrapper4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                        
        
     
    #</methodize4xi>
                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "recv-of-ConnectionWrapper4Xi" usage = "receives with (self)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def recv(self):
        '''
        ConnectionWrapper4Xi recv method receives with (self)
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)          
        # -----------------------------------------------------------------------------------------
                 
        class2print = 'ConnectionWrapper4Xi'
        method2print = "recv"
        
        
        message2print = "{(Step 1/3) started" + "; recv-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)            
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/3a) calling" + "; conn-recv-bytes-of-connectionwrapper2xi-calling-by-recv-of-ConnectionWrapper4Xi: " + str(self._conn.recv_bytes) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
                        
        
        s2xi = self._conn.recv_bytes()
        
        message2print = "{(Step 2/3b) set" + "; s2xi-set-by-recv-of-ConnectionWrapper4Xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
                        


        # -----------------------------------------------------------------------------------------
        # (Step 3/3)            
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3/3a) passing" + "; s2xi-passing-to-loads-of-connectionwrapper2xi-by-recv-of-ConnectionWrapper4Xi: " + str(s2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        message2print = "{(Step 3/3b) returning" + "; loads-function-of-connectionwrapper2xi-returning-by-recv-of-ConnectionWrapper4Xi: " + str(self._loads) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                                        
                
        return self._loads(s2xi)
    #</methodize4xi>
                                          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ConnectionWrapper4Xi" usage = "senses class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses class objects
        '''
          
        
        class2print = 'ConnectionWrapper4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ConnectionWrapper4Xi-sensing-by-sense-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ConnectionWrapper4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['inheritance-by-connectionwrapper2xi'] = object
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-ConnectionWrapper4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
       
        
      
        message2print = "{sensed" + "; ConnectionWrapper4Xi-sensed-by-sense-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ConnectionWrapper4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                              
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ConnectionWrapper4Xi" usage = "reasons class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons class objects
        '''
                 
        
        class2print = 'ConnectionWrapper4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ConnectionWrapper4Xi-reasoning-by-reason-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ConnectionWrapper4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; ConnectionWrapper4Xi-reasoned-by-reason-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ConnectionWrapper4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                   
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ConnectionWrapper4Xi" usage = "">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces class objects 
        '''
               
        
        
        class2print = 'ConnectionWrapper4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-ConnectionWrapper4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ConnectionWrapper4Xi-acing-by-ace-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                   
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ConnectionWrapper4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; ConnectionWrapper4Xi-aced-by-ace-of-ConnectionWrapper4Xi: " + str(ConnectionWrapper4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ConnectionWrapper4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of connectionwrapper4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



